with Ada.Numerics.Discrete_Random;

package body room is

   Room_Height : Positive;
   Room_Width  : Positive;
   Room_Doors  : boolean_array_type(1..max_doors) := (others => False);
   
   -----------------
   -- create_room --
   -----------------

   procedure create (R : in out room_type; height : Positive; width : Positive; num_doors : Natural) is
      subtype Index_Type is Positive range 1..2*height+2*width;
      package RandomDoor is new Ada.Numerics.Discrete_Random (Result_Subtype => Index_Type);
      
      D       : Index_Type;
      G       : RandomDoor.Generator;
      Pdiv2   : Positive := width + height;
   begin
      Room_Height := height;
      R.height    := height;
      Room_Width  := width;
      R.width     := width;
      R.perimeter := Pdiv2 * 2;
      
      RandomDoor.Reset (Gen => G);
      
      for I in 1..num_doors loop
         loop
            D := RandomDoor.Random(Gen=> G);
            
            if not R.doors(D) then
               if D =       1 or D =       R.width or D =       R.width+1 or D =       Pdiv2 or
                  D = Pdiv2+1 or D = Pdiv2+R.Width or D = Pdiv2+R.width+1 or D = Pdiv2+Pdiv2 then
                     null;
               else
                     R.doors(D)    := True;
                     Room_Doors(D) := True;
                     exit;
               end if;
            end if;
         end loop;
      end loop;
   end create;

   -----------------
   -- create_room --
   -----------------

   procedure create (R : in out room_type; height : Positive; width : Positive; doors : boolean_array_type)
   is
   begin
      Room_Height := height;
      R.height    := height;
      Room_Width  := width;
      R.width     := width;
      
      R.doors    := doors;
      Room_Doors := doors;
   end create;

   ---------------
   -- draw_tile --
   ---------------

   procedure draw_tile (X : Natural; Y : Natural) is
   begin
            Adagraph.Draw_Box(X1 => (X-1) * Tile_Size + Offset, 
                              Y1 => (Y-1) * Tile_Size + Offset, 
                              X2 => X * Tile_Size + Offset, 
                              Y2 => Y * Tile_Size + Offset,
                              Hue => Tile_Color(X => X, Y => Y), Filled => True);
   end draw_tile;
      
   ----------
   -- draw --
   ----------

   procedure draw (R: room_type) is
      X1, Y1    : Integer;
      X2, Y2    : Integer;
      Pdiv2     : Positive := R.width + R.height;
      P         : Positive := R.perimeter;
   begin
      for X in 0..R.width-1 loop
         for y in 0..R.height-1 loop
            Adagraph.Draw_Box(X1 => X * Tile_Size + Offset, 
                              Y1 => Y * Tile_Size + Offset, 
                              X2 => (X+1) * Tile_Size + Offset, 
                              Y2 => (Y+1) * Tile_Size + Offset,
                              Hue => Tile_Color(X => X + 1, Y => Y + 1), Filled => True);
         end loop;
      end loop;
      
      for I in 1..R.perimeter loop
      
         if R.doors(I) then
            if                  1 <= I and I <=       R.width then
               X1 := (I - 1)  * Tile_Size;
               X2 := I * Tile_Size;
               Y1 := R.height * Tile_Size; 
               Y2 := R.height * Tile_Size + Door_Width;
            elsif       R.width+1 <= I and I <=         Pdiv2 then
               X1 := R.width * Tile_Size;
               X2 := R.width * Tile_Size + Door_Width;
               Y1 := (Pdiv2 - I) * Tile_Size;
               Y2 := (Pdiv2 - I + 1) * Tile_Size;
            elsif         Pdiv2+1 <= I and I <= Pdiv2+R.width then
               X1 := (R.width - (I - Pdiv2)) * Tile_Size;
               X2 := (R.width - (I - Pdiv2) + 1) * Tile_Size;
               Y1 := 0;
               Y2 := - Door_Width;
            elsif Pdiv2+R.width+1 <= I and I <=   Pdiv2+Pdiv2 then
               X1 := 0;
               X2 := - Door_Width;
               Y1 := (R.height - (P - I) - 1) * Tile_Size;
               Y2 := (R.height - (P - I)) * Tile_Size;
            end if;
      
      
            Adagraph.Draw_Box(X1 => X1 + Offset, 
                              Y1 => Y1 + Offset, 
                              X2 => X2 + Offset, 
                              Y2 => Y2 + Offset, Hue => Adagraph.Light_Blue, Filled => True);
         end if;
      end loop;
   end draw;

   -------------
   -- Is_Door --
   -------------

   function Is_Door (X : Positive; Y : Positive; direction : Common.Direction_Type)
      return Boolean
   is
      offset : Integer;
   begin
      case direction is
         when Common.north =>
            if Y = Room_Height then
               return Room_Doors(X);
            end if;
         when Common.south =>
            if Y = 1 then
               offset := 2 * Room_Width + Room_Height;
               return Room_Doors(offset - X);
            end if;
         when Common.east  =>
            if X = Room_Width then
               offset := Room_Width + Room_Height + 1;
               return Room_Doors(offset - Y);
            end if;
         when Common.west  =>
            if X = 1 then
               offset := 2 * Room_Width + Room_Height;
               return Room_Doors(offset + Y);
            end if;
      end case;
      return False;
   end Is_Door;

   ----------------
   -- Tile_Color --
   ----------------

   function Tile_Color (X : Positive; Y : Positive)
      return Adagraph.Extended_Color_Type
   is
   begin
      if Y mod 2 = X mod 2 then
         return Adagraph.Light_Gray;
      else
         return Adagraph.Dark_Gray;
      end if;
   end Tile_Color;

   -----------------
   -- Room_Height --
   -----------------

   function Height return Positive is
   begin
      return Room_Height;
   end Height;

   ----------------
   -- Room_Width --
   ----------------

   function Width return Positive is
   begin
      return Room_Width;
   end Width;

   ----------------------
   -- Tile_To_Screen_X --
   ----------------------

   function Tile_To_Screen_X (X : Natural) return Natural is
   begin
      return (X - 1) * Tile_Size + Tile_Size / 2 + Offset;
   end Tile_To_Screen_X;

   ----------------------
   -- Tile_To_Screen_Y --
   ----------------------

   function Tile_To_Screen_Y (Y : Natural) return Natural is
   begin
      return (Y - 1) * Tile_Size + Tile_Size / 2 + Offset;
   end Tile_To_Screen_Y;

end room;

